#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>

#include <time.h>
#include <sys/time.h>

#include "mma8452.h"

#define MAIN_C
#include "exitfail.h"

#define BASENAME(p) ((strrchr((p), '/') ? : ((p)-1))+1)

#define I2C_ADDR	0x1c /* Device Address(when SA0=0) 0011100=0x1c */

static void usage(char *prog)
{
	printf("Usage: %s <-d|--device FILENAME>\n", BASENAME(prog));
}

static void parse_arg(int argc, char *argv[], char **device)
{
	int c;

	*device = NULL;
	for(;;) {
		int option_index = 0;
		static struct option long_options[] = {
			{"device",	required_argument,	NULL,	'd'},
			{0,		0,			0,	0},
		};

		c = getopt_long(argc, argv, "d:",
				  long_options, &option_index);

		if(c == -1) break;
		
		switch(c) {
		case 'd':
			*device = optarg;
			if(errno != 0) 
				goto err;
			break;
		default:
			goto err;
		}
	}

	if(*device == NULL) goto err;
	return;

err:
	usage(argv[0]);
	exit(EXIT_FAILURE);
}

int c2toi_12bit(int c2){
	int ret;

	if(c2 > 0x800) ret = -(0x1000 - c2);
	else ret = c2;

	return ret;
}

int main(int argc, char *argv[])
{
	struct mma8452 *acc;
	char *device;

	uint8_t digital_code[64];
	int i;
	int ret;

    	struct timeval time;

	/**** MMA8452: variable : begin ****/
	int xc, yc, zc, xi, yi, zi;
	/**** MMA8452: variable: end ****/

	exitfail_init();

	parse_arg(argc, argv, &device);

	acc = mma8452_open(device, I2C_ADDR);
	if(acc == NULL) exitfail_errno("mma8452_open");

	for(i = 0; i < 64; i++) digital_code[i] = 0x00;

	/**** MMA8452: SMBUS Read Register(WHO_AM_I: 0x0d): begin ****/
	ret = mma8452_read_smbus_byte(acc, 0x0d, digital_code);
	if(ret !=0) exitfail_errno("mma8452_read_smbus_byte");
	printf("-------- WHO_AM_I --------\n");
	printf("WHO_AM_I(0x%02X): 0x%02X\n", 0x0d, digital_code[0]);
	/**** MMA8452: SMBUS Read Register(WHO_AM_I: 0x0d): end ****/

	/**** MMA8452: SMBUS Read Register(All: 0x00-0x31): begin ****/
	printf("-------- Read All Registers --------\n");
	for(i = 0; i < 0x31 + 1; i++) {
		ret = mma8452_read_smbus_byte(acc, i, digital_code);
		if(ret !=0) exitfail_errno("mma8452_read_smbus_byte");
		printf("0x%02X: 0x%02X\n", i, digital_code[0]);
	}
	/**** MMA8452: SMBUS Read Register(0x00-0x31): end ****/

	/**** MMA8452: Write Register(CTRL_REG1: 0x2A) for ACTIVE: begin ****/
	digital_code[0] = 0x01; /* CTRL_REG1[0] = 1: ACTIVE */
	ret = mma8452_write(acc, 0x2A, 1, digital_code);
	if(ret !=0) exitfail_errno("mma8452_write");

	ret = mma8452_read_smbus_byte(acc, 0x2a, digital_code);
	if(ret !=0) exitfail_errno("mma8452_read_smbus_byte");
	printf("-------- Set CTRL_REG1[0] = 1: ACTIVE --------\n");
	printf("CTRL_REG1(0x%02X): 0x%02X\n", 0x2a, digital_code[0]);
	/**** MMA8452: Read Register(CTRL_REG1: 0x2A) for ACTIVE: end ****/

	/**** MMA8452: Read Register(Axis): begin ****/
	while(1) {
	    	gettimeofday(&time, NULL);
		ret = mma8452_read(acc, 0x00, 7, digital_code);
		if(ret !=0) exitfail_errno("mma8452_read");

		printf("-------- Get Status and XYZ-Axis --------\n");
	    	printf("time=%d[s](%d[us])\n", (int)time.tv_sec, (int)time.tv_usec);
		printf("STATUS(0x%02X): 0x%02X\n", 0, digital_code[0]);

		xc = ((digital_code[1] << 4) & 0xFF0) + ((digital_code[2] >> 4) & 0x00F);
		yc = ((digital_code[3] << 4) & 0xFF0) + ((digital_code[4] >> 4) & 0x00F);
		zc = ((digital_code[5] << 4) & 0xFF0) + ((digital_code[6] >> 4) & 0x00F);
		xi = c2toi_12bit(xc);
		yi = c2toi_12bit(yc);
		zi = c2toi_12bit(zc);

		printf("X: 0x%03X = %6d\n", xc, xi);
		printf("Y: 0x%03X = %6d\n", yc, yi);
		printf("Z: 0x%03X = %6d\n", zc, zi);
	
		usleep(100000);
	}
	/**** MMA8452: Read Register(Axis): end ****/

	ret = mma8452_close(acc);

	if(ret != 0) exitfail_errno("mma8452_close");

	return EXIT_SUCCESS;
}
