#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/i2c-dev.h>

#include <stdio.h> 
#include <time.h> 

/**** sample: for I2C SMBUS: begin ****/
#include <linux/i2c.h>
/**** sample: for I2C SMBUS: end ****/

#include "mma8452.h"

#define MMA8452_RETRY_MAX 3
#define MMA8452_REG_MIN 0x00
#define MMA8452_REG_MAX 0x31

struct mma8452 {
	int fd;
};

static ssize_t write_uninterruptible(int fd, const void *buf, size_t count, int retry)
{
	ssize_t ret;
	int i;

	for(i=0; i<retry; i++) {
		ret = write(fd, buf, count);
		if(ret < 0 && errno != EINTR) return -1;
		if((size_t)ret == count) return ret;
	}
	errno = ETIMEDOUT;
	return -1;
}

static ssize_t read_uninterruptible(int fd, void *buf, size_t count, int retry)
{
	size_t read_length = 0;
	ssize_t ret;
	int i;

	for(i=0; i<retry; i++) {
		ret = read(fd, buf+read_length, count-read_length);
		if(ret < 0) {
			if(errno == EINTR) continue;
			else return -1;
		}

		read_length += ret;
		if(read_length == count) return read_length;
	}

	errno = ETIMEDOUT;
	return -1;
}

struct mma8452 *mma8452_open(const char *dev_path, const int addr)
{
	struct mma8452 *acc;
	int error;
	int ret;

	if(dev_path == NULL) {
		errno = EINVAL;
		return NULL;
	}

	acc = calloc(1, sizeof(struct mma8452));
	if(acc == NULL) return NULL;

	acc->fd = open(dev_path, O_RDWR);
	if(acc->fd < 0) {
		error = errno;
		goto err1;
	}

	ret = ioctl(acc->fd, I2C_SLAVE, addr);
	if(ret < 0) {
		error = errno;
		goto err2;
	}

	return acc;

err2:
	close(acc->fd);
err1:
	free(acc);
	errno = error;
	return NULL;
}

int mma8452_write(struct mma8452 *acc, uint8_t start_addr, int write_size, uint8_t *digit)
{
	int i;
	int ret;
	uint8_t buf[MMA8452_REG_MAX + 1];

	if(acc == NULL || digit == NULL || 
		start_addr + write_size > MMA8452_REG_MAX + 1) {
		errno = EINVAL;
		return -1;
	}

	buf[0] = start_addr;
	for(i = 0; i < write_size; i++) buf[i + 1] = digit[i];

	ret = write_uninterruptible(acc->fd, buf, write_size + 1, MMA8452_RETRY_MAX);
	if(ret < 0) return -1;

	return 0;
}

int mma8452_read(struct mma8452 *acc, uint8_t start_addr, int read_size, uint8_t *digit)
{
	int ret;

	if(acc == NULL || digit == NULL || 
		start_addr + read_size > MMA8452_REG_MAX + 1) {
		errno = EINVAL;
		return -1;
	}

	ret = write_uninterruptible(acc->fd, &start_addr, 1, MMA8452_RETRY_MAX);
	if(ret < 0) return -1;

	ret = read_uninterruptible(acc->fd, digit, read_size, MMA8452_RETRY_MAX);
	if(ret < 0) return -1;

	return 0;
}

/**** sample: for I2C SMBUS: begin ****/
int mma8452_read_smbus_byte(struct mma8452 *acc, uint8_t start_addr, uint8_t *digit)
{
	union i2c_smbus_data data;
	struct i2c_smbus_ioctl_data args;

	if(acc == NULL || digit == NULL || 
		start_addr > MMA8452_REG_MAX) {
		errno = EINVAL;
		return -1;
	}

	args.read_write = I2C_SMBUS_READ;
	args.command = start_addr;
	args.size = I2C_SMBUS_BYTE_DATA;
	args.data = &data;

	if (ioctl(acc->fd, I2C_SMBUS, &args))
		return -1;
	else
		*digit = 0xFF & data.byte;

	return 0;
}
/**** sample: for I2C SMBUS: end ****/

int mma8452_close(struct mma8452 *acc)
{
	int fd;

	if(acc == NULL) {
		errno = EINVAL;
		return -1;
	}

	fd = acc->fd;
	free(acc);

	return close(fd);
}

